@@ -14,7 +14,7 @@
 #' @param tile a character verctor with a map tiles, popularized by Google Maps. See \href{https://leaflet-extras.github.io/leaflet-providers/preview/index.html}{here} for the complete set.
 #' @param glottolog.source A character vector that define which glottolog database is used: "original" or "modified" (by default)
 #' @param color vector of colors or palette. The color argument can be (1) a character vector of RGM or named colors; (2) the name of an RColorBrewer palette; (3) the full name of a viridis palette; (4) a function that receives a single value between 0 and 1 and returns a color. For more examples see \code{\link{colorNumeric}}
-#' @param control logical. If TRUE, function show layer control buttons. By default is TRUE.
+#' @param control logical. If TRUE, function show layer control buttons. By default is FALSE
 #' @param density.estimation.color vector of density polygons' colors
 #' @param density.estimation.opacity a numeric vector of density polygons opacity.
 #' @param density.latitude.width bandwidths for latitude values. Defaults to normal reference bandwidth (see \link{bandwidth.nrd}).
@@ -24,6 +24,7 @@
 #' @param density.longitude.width bandwidths for longitude values. Defaults to normal reference bandwidth (see \link{bandwidth.nrd}).
 #' @param density.points logical. If FALSE, it doesn't show points in polygones.
 #' @param density.title title of a density-feature legend
+#' @param density.control logical. If TRUE, function show layer control buttons for density plot. By default is FALSE
 #' @param image.height numeric vector of image heights
 #' @param image.url character vector of URLs with an images
 #' @param image.width numeric vector of image widths
@@ -32,6 +33,7 @@
 #' @param label.fsize numeric value of the label font size. By default is 14.
 #' @param label.hide logical. If FALSE, labels are displayed allways. If TRUE, labels are displayed on mouse over. By default is TRUE.
 #' @param label.position the position of labels: "left", "right", "top", "bottom"
+#' @param label.emphasize is the list. First argument is a vector of points in datframe that should be emphasized. Second argument is a string with a color for emphasis.
 #' @param legend logical. If TRUE, function show legend. By default is FALSE.
 #' @param legend.opacity a numeric vector of legend opacity.
 #' @param legend.position the position of the legend: "topright", "bottomright", "bottomleft","topleft"
@@ -39,8 +41,12 @@
 #' @param minimap.height The height of the minimap in pixels.
 #' @param minimap.position the position of the minimap: "topright", "bottomright", "bottomleft","topleft"
 #' @param minimap.width The width of the minimap in pixels.
+#' @param minichart citation from leaflet.minicharts package: "Possible values are "bar" for bar charts, "pie" for pie charts, "polar-area" and "polar-radius"."
+#' @param minichart.data citation from leaflet.minicharts package: "A numeric matrix with number of rows equal to the number of elements in lng or lat and number of column equal to the number of variables to represent. If parameter time is set, the number of rows must be equal to the length of lng times the number of unique time steps in the data."
+#' @param minichart.time citation from leaflet.minicharts package: "A vector with length equal to the number of rows in chartdata and containing either numbers representing time indices or dates or datetimes. Each unique value must appear as many times as the others. This parameter can be used when one wants to represent the evolution of some variables on a map."
+#' @param minichart.labels citation from leaflet.minicharts package: "Should values be displayed above chart elements."
 #' @param opacity a numeric vector of marker opacity.
-#' @param radius a numeric vector of radii for the circles.
+#' @param width a numeric vector of radius for circles or width for barcharts in minicharts.
 #' @param scale.bar logical. If TRUE, function shows scale-bar. By default is TRUE.
 #' @param scale.bar.position the position of the scale-bar: "topright", "bottomright", "bottomleft","topleft"
 #' @param stroke.color vector of stroke colors
@@ -88,14 +94,12 @@
 #' map.feature(df$lang, df$feature, df$popup,
 #' stroke.features = df$popup)
 #'
-#'
 #' ## Add a minimap to plot
 #' map.feature(c("Adyghe", "Russian"), minimap = TRUE)
 #'
 #' ## Remove scale bar
 #' map.feature(c("Adyghe", "Russian"), scale.bar = FALSE)
 #'
-#'
 #' @export
 #' @importFrom leaflet colorNumeric
 #' @importFrom leaflet colorFactor
@@ -118,6 +122,8 @@
 #' @importFrom grDevices topo.colors
 #' @importFrom rowr cbind.fill
 #' @importFrom magrittr %>%
+#' @importFrom leaflet.minicharts addMinicharts
+#' @importFrom leaflet.minicharts popupArgs
 #'
 
 map.feature <- function(languages,
@@ -129,6 +135,7 @@ map.feature <- function(languages,
                         label.hide = TRUE,
                         label.fsize = 14,
                         label.position = "right",
+                        label.emphasize = list(NULL, "black"),
                         stroke.features = NULL,
                         density.estimation = NULL,
                         density.estimation.color = NULL,
@@ -140,6 +147,7 @@ map.feature <- function(languages,
                         density.legend.opacity = 1,
                         density.legend.position = "bottomleft",
                         density.title = "",
+                        density.control = FALSE,
                         color = NULL,
                         stroke.color = NULL,
                         image.url = NULL,
@@ -156,7 +164,7 @@ map.feature <- function(languages,
                         stroke.legend = TRUE,
                         stroke.legend.opacity = 1,
                         stroke.legend.position = "bottomleft",
-                        radius = 5,
+                        width = 5,
                         stroke.radius = 9.5,
                         opacity = 1,
                         stroke.opacity = 1,
@@ -172,28 +180,41 @@ map.feature <- function(languages,
                         zoom.level = NULL,
                         rectangle.lng = NULL,
                         rectangle.lat = NULL,
+                        minichart = NULL,
+                        minichart.data = NULL,
+                        minichart.time = NULL,
+                        minichart.labels = FALSE,
                         rectangle.color = "black",
                         map.orientation = "Pacific",
-                        glottolog.source = "modified"){
-
-  ifelse(grepl(glottolog.source, "original"),
-         glottolog <- lingtypology::glottolog.original,
-         glottolog <- lingtypology::glottolog.modified)
-  if(typeof(languages) == "list"){languages <- unlist(languages)}
-  if(sum(is.glottolog(languages,
-                      response = TRUE,
-                      glottolog.source = glottolog.source)) == 0){
+                        glottolog.source = "modified") {
+  ifelse(
+    grepl(glottolog.source, "original"),
+    glottolog <- lingtypology::glottolog.original,
+    glottolog <- lingtypology::glottolog.modified
+  )
+  if (typeof(languages) == "list") {
+    languages <- unlist(languages)
+  }
+  if (sum(is.glottolog(
+    languages,
+    response = TRUE,
+    glottolog.source = glottolog.source
+  )) == 0) {
     stop("There is no data to map")
   }
 
   # create dataframe ---------------------------------------------------------
   mapfeat.df <- data.frame(languages, features,
                            popup = popup)
-  if(sum(label == "") != length(label)){
+  if (sum(label == "") != length(label)) {
     mapfeat.df$label <- as.character(label)
+    if (!is.null(label.emphasize[[1]])) {
+      mapfeat.df$emph <- "emph"
+      mapfeat.df$emph[-label.emphasize[[1]]] <- "no"
+    }
   }
 
-  if(!is.null(density.estimation)){
+  if (!is.null(density.estimation)) {
     mapfeat.df$density.estimation <- density.estimation
   }
 
@@ -203,136 +224,183 @@ map.feature <- function(languages,
     mapfeat.df$long <- long.lang(languages,
                                  map.orientation = map.orientation,
                                  glottolog.source = glottolog.source)
-    mapfeat.df$lat <- lat.lang(languages, glottolog.source = glottolog.source)
-  } else {   # if there are latitude and longitude
+    mapfeat.df$lat <-
+      lat.lang(languages, glottolog.source = glottolog.source)
+  } else {
+    # if there are latitude and longitude
     mapfeat.df$long <- longitude
     mapfeat.df$lat <- latitude
   }
 
   # if there are no coordinates... ------------------------------------------
-  if(sum(is.na(mapfeat.df$long&mapfeat.df$lat))> 0){
-    warning(paste("There is no coordinates for languages",
-                  paste(
-                    mapfeat.df$languages[is.na(mapfeat.df$long&mapfeat.df$lat)],
-                    collapse = ", ")),
-            call. = FALSE)
+  if (sum(is.na(mapfeat.df$long & mapfeat.df$lat)) > 0) {
+    warning(paste(
+      "There is no coordinates for languages",
+      paste(mapfeat.df$languages[is.na(mapfeat.df$long &
+                                         mapfeat.df$lat)],
+            collapse = ", ")
+    ),
+    call. = FALSE)
   }
 
   # remove any rows with NAs ------------------------------------------------
-  mapfeat.df <- mapfeat.df[stats::complete.cases(mapfeat.df),]
+  mapfeat.df <- mapfeat.df[stats::complete.cases(mapfeat.df), ]
 
   # create link --------------------------------------------------------------
-  mapfeat.df$link <- url.lang(as.character(mapfeat.df$languages),
-                              popup = mapfeat.df$popup,
-                              glottolog.source = glottolog.source)
+  mapfeat.df$link <- url.lang(
+    as.character(mapfeat.df$languages),
+    popup = mapfeat.df$popup,
+    glottolog.source = glottolog.source
+  )
 
   # add images --------------------------------------------------------------
-  if(!is.null(image.url)){
-    mapfeat.image <- rowr::cbind.fill(mapfeat.df[,-2], data.frame(image.url))
-    mapfeat.image <- mapfeat.image[stats::complete.cases(mapfeat.image),]
+  if (!is.null(image.url)) {
+    mapfeat.image <-
+      rowr::cbind.fill(mapfeat.df[, -2], data.frame(image.url))
+    mapfeat.image <-
+      mapfeat.image[stats::complete.cases(mapfeat.image), ]
   }
 
   # create a stroke dataframe -----------------------------------------------
-  if(!is.null(stroke.features)){
-    mapfeat.stroke <- rowr::cbind.fill(mapfeat.df[,-2],
+  if (!is.null(stroke.features)) {
+    mapfeat.stroke <- rowr::cbind.fill(mapfeat.df[, -2],
                                        data.frame(stroke.features))
-    mapfeat.stroke <- mapfeat.stroke[stats::complete.cases(mapfeat.stroke),]
+    mapfeat.stroke <-
+      mapfeat.stroke[stats::complete.cases(mapfeat.stroke), ]
   }
 
   # create a palette ---------------------------------------------------------
-  my_colors <- grDevices::colors()[!grepl("ivory|azure|white|gray|grey|black|pink|1", grDevices::colors())]
+  set.seed(45)
+  my_colors <-
+    c(
+      "#1f77b4",
+      "#ff7f0e",
+      "#2ca02c",
+      "#d62728",
+      "#9467bd",
+      "#8c564b",
+      "#e377c2",
+      "#7f7f7f",
+      "#17becf",
+      sample(grDevices::colors()[!grepl("ivory|azure|white|gray|grey|black|pink|1",
+                                        grDevices::colors())])
+    )
   if (is.null(color)) {
-    if(is.numeric(mapfeat.df$features)){
-      pal <- leaflet::colorNumeric(palette = "BuPu", domain=mapfeat.df$features)
-    } else {set.seed(45)
-      pal <- leaflet::colorFactor(
-      sample(my_colors, length(unique(mapfeat.df$features))),
-      domain = mapfeat.df$features)
-    }} else {
-      if(is.numeric(mapfeat.df$features)){
-        pal <- leaflet::colorNumeric(palette = color,domain=mapfeat.df$features)
-      }else {
-        if(length(mapfeat.df$features) == length(color)){
-           df <- unique(data.frame(feature = mapfeat.df$features, color))
-           color <- as.character(df[order(df$feature),]$color)
-           }
-        pal <- leaflet::colorFactor(color, domain = mapfeat.df$features)
-      }
+    if (is.numeric(mapfeat.df$features)) {
+      pal <-
+        leaflet::colorNumeric(palette = "BuPu", domain = mapfeat.df$features)
+    } else {
+      pal <-
+        leaflet::colorFactor(my_colors[1:length(unique(mapfeat.df$features))],
+                             domain = mapfeat.df$features)
     }
+  } else {
+    if (is.numeric(mapfeat.df$features)) {
+      pal <-
+        leaflet::colorNumeric(palette = color, domain = mapfeat.df$features)
+    } else {
+      if (length(mapfeat.df$features) == length(color)) {
+        df <- unique(data.frame(feature = mapfeat.df$features, color))
+        color <- as.character(df[order(df$feature), ]$color)
+      }
+      pal <-
+        leaflet::colorFactor(color, domain = mapfeat.df$features)
+    }
+  }
 
-    if (is.null(density.estimation.color)) {
-    if(is.numeric(mapfeat.df$density.estimation)){
-      density.estimation.pal <- leaflet::colorNumeric(palette = "BuPu", domain=mapfeat.df$density.estimation)
+  if (is.null(density.estimation.color)) {
+    if (is.numeric(mapfeat.df$density.estimation)) {
+      density.estimation.pal <-
+        leaflet::colorNumeric(palette = "BuPu",
+                              domain = mapfeat.df$density.estimation)
     } else {
       set.seed(45)
-      density.estimation.pal <- leaflet::colorFactor(
-      sample(my_colors, length(unique(mapfeat.df$density.estimation)),
-             length(unique(mapfeat.df$density.estimation))),
-      domain = mapfeat.df$density.estimation)
-    }} else {
-        if(length(mapfeat.df$density.estimation) == length(density.estimation.color)){
-          df <- unique(data.frame(feature = mapfeat.df$density.estimation, color = density.estimation.color))
-          density.estimation.color <- as.character(df[order(df$feature),]$color)
-          }
-        density.estimation.pal <- leaflet::colorFactor(unique(density.estimation.color), domain = mapfeat.df$density.estimation)
+      density.estimation.pal <-
+        leaflet::colorFactor(c(rev(my_colors[1:9]), my_colors[10:329])[1:length(unique(mapfeat.df$density.estimation))],
+                             domain = mapfeat.df$density.estimation)
     }
+  } else {
+    if (length(mapfeat.df$density.estimation) == length(density.estimation.color)) {
+      df <-
+        unique(
+          data.frame(
+            feature = mapfeat.df$density.estimation,
+            color = density.estimation.color
+          )
+        )
+      density.estimation.color <-
+        as.character(df[order(df$feature), ]$color)
+    }
+    density.estimation.pal <-
+      leaflet::colorFactor(unique(density.estimation.color),
+                           domain = mapfeat.df$density.estimation)
+  }
 
-  if(!is.null(stroke.features)){
+  if (!is.null(stroke.features)) {
     if (is.null(stroke.color)) {
-      lev <- nlevels(as.factor(
-        stroke.features[stats::complete.cases(stroke.features)]))
-      strokecolors <- grDevices::gray(lev :0 / lev)
+      lev <-
+        nlevels(as.factor(stroke.features[stats::complete.cases(stroke.features)]))
+      strokecolors <- grDevices::gray(lev:0 / lev)
       stroke.pal <- leaflet::colorFactor(strokecolors,
-                                        domain = mapfeat.stroke$stroke.features)
+                                         domain = mapfeat.stroke$stroke.features)
       rev.stroke.pal <- leaflet::colorFactor(rev(strokecolors),
-                                        domain = mapfeat.stroke$stroke.features)
+                                             domain = mapfeat.stroke$stroke.features)
     } else {
       stroke.pal <- leaflet::colorFactor(stroke.color,
-                                        domain = mapfeat.stroke$stroke.features)
+                                         domain = mapfeat.stroke$stroke.features)
       rev.stroke.pal <- leaflet::colorFactor(rev(stroke.color),
-                                        domain = mapfeat.stroke$stroke.features)
+                                             domain = mapfeat.stroke$stroke.features)
     }
   }
 
   # change tile names if needed ---------------------------------------------
   ifelse(is.null(tile.name), tile.name <- tile, NA)
-  if(length(tile.name) != length(tile)){
+  if (length(tile.name) != length(tile)) {
     tile.name <- tile
     ifelse(length(tile.name) > length(tile),
-           warning('number of tiles (tile argument) is less than number of tile names (tile.name argument)', call. = FALSE),
-           warning('number of tile names (tile.name argument) is less than number of tiles (tile argument)', call. = FALSE))
+           warning(
+             'number of tiles (tile argument) is less than number of tile names (tile.name argument)',
+             call. = FALSE
+           ),
+           warning(
+             'number of tile names (tile.name argument) is less than number of tiles (tile argument)',
+             call. = FALSE
+           ))
   }
 
   # create a density polygones --------------------------------------------
-  if(!is.null(density.estimation)){
-    my_poly_names <- names(which(table(mapfeat.df$density.estimation) > 1))
-    my_poly <- lapply(my_poly_names, function(feature){
-      polygon.points(mapfeat.df[mapfeat.df$density.estimation == feature, 'lat'],
-                     mapfeat.df[mapfeat.df$density.estimation == feature, 'long'],
-                     latitude_width = density.latitude.width,
-                     longitude_width = density.longitude.width)
-  })
+  if (!is.null(density.estimation)) {
+    my_poly_names <-
+      names(which(table(mapfeat.df$density.estimation) > 1))
+    my_poly <- lapply(my_poly_names, function(feature) {
+      polygon.points(
+        mapfeat.df[mapfeat.df$density.estimation == feature, 'lat'],
+        mapfeat.df[mapfeat.df$density.estimation == feature, 'long'],
+        latitude_width = density.latitude.width,
+        longitude_width = density.longitude.width
+      )
+    })
   }
 
   ### create a map ------------------------------------------------------------
-  m <- leaflet::leaflet(mapfeat.df, option=leaflet::leafletOptions(
-    zoomControl = zoom.control)) %>%
+  m <- leaflet::leaflet(mapfeat.df,
+                        option = leaflet::leafletOptions(zoomControl = zoom.control)) %>%
     leaflet::addTiles(tile[1]) %>%
     leaflet::addProviderTiles(tile[1], group = tile.name[1])
-  if(length(tile) > 1){
-    mapply(function(other.tiles, other.tile.names){
+  if (length(tile) > 1) {
+    mapply(function(other.tiles, other.tile.names) {
       m <<- m %>% leaflet::addProviderTiles(other.tiles,
                                             group = other.tile.names)
     }, tile[-1], tile.name[-1])
   }
 
   # map: add rectangle ------------------------------------------------------
-  if (!is.null(rectangle.lng)&!is.null(rectangle.lat)) {
+  if (!is.null(rectangle.lng) & !is.null(rectangle.lat)) {
     m <- m %>% leaflet::addRectangles(
-      lng1= rectangle.lng[1],
-      lat1=rectangle.lat[1],
-      lng2=rectangle.lng[2],
-      lat2=rectangle.lat[2],
+      lng1 = rectangle.lng[1],
+      lat1 = rectangle.lat[1],
+      lng2 = rectangle.lng[2],
+      lat2 = rectangle.lat[2],
       color = rectangle.color,
       opacity = 1,
       weight = 3,
@@ -341,133 +409,258 @@ map.feature <- function(languages,
   }
 
   # if there is density estimation ------------------------------------------
-  if(!is.null(density.estimation)){
-    lapply(seq_along(my_poly), function(x){
-      m <<- m %>% leaflet::addPolygons(data = my_poly[[x]],
-                                       color = density.estimation.pal(my_poly_names[x]),
-                                       opacity = 0.2,
-                                       fillOpacity=density.estimation.opacity)})
+  if (!is.null(density.estimation)) {
+    lapply(seq_along(my_poly), function(x) {
+      m <<- m %>% leaflet::addPolygons(
+        data = my_poly[[x]],
+        color = density.estimation.pal(my_poly_names[x]),
+        opacity = 0.2,
+        fillOpacity = density.estimation.opacity,
+        group = my_poly_names[x]
+      )
+    })
   }
 
-    # map: if there are stroke features ---------------------------------------
-  if(!is.null(stroke.features)){
-    m <- m %>% leaflet::addCircleMarkers(lng=mapfeat.stroke$long,
-                                         lat=mapfeat.stroke$lat,
-                                         popup= mapfeat.stroke$link,
-                                         stroke = FALSE,
-                                         radius = stroke.radius*1.15,
-                                         fillOpacity = stroke.opacity,
-                                         color = "black") %>%
-leaflet::addCircleMarkers(lng=mapfeat.stroke$long,
-                  lat=mapfeat.stroke$lat,
-                  popup= mapfeat.stroke$link,
-                  stroke = FALSE,
-                  radius = stroke.radius,
-                  fillOpacity = stroke.opacity,
-                  color=stroke.pal(mapfeat.stroke$stroke.features),
-                  group = mapfeat.stroke$stroke.features) %>%
-leaflet::addCircleMarkers(lng=mapfeat.stroke$long,
-                    lat=mapfeat.stroke$lat,
-                    popup= mapfeat.df$link,
-                    labelOptions = leaflet::labelOptions(noHide = !(label.hide),
-                        direction = label.position,
-                        textOnly = TRUE,
-                        style = list("font-size" = paste0(label.fsize, "px"))),
-                    stroke = FALSE,
-                    radius = 1.15*radius,
-                    fillOpacity = opacity,
-                    color = rev.stroke.pal(mapfeat.stroke$stroke.features),
-                    group = mapfeat.stroke$stroke.features)}
+  # map: if there are stroke features ---------------------------------------
+  if (!is.null(stroke.features)) {
+    m <- m %>% leaflet::addCircleMarkers(
+      lng = mapfeat.stroke$long,
+      lat = mapfeat.stroke$lat,
+      popup = mapfeat.stroke$link,
+      stroke = FALSE,
+      radius = stroke.radius * 1.15,
+      fillOpacity = stroke.opacity,
+      color = "black"
+    ) %>%
+      leaflet::addCircleMarkers(
+        lng = mapfeat.stroke$long,
+        lat = mapfeat.stroke$lat,
+        popup = mapfeat.stroke$link,
+        stroke = FALSE,
+        radius = stroke.radius,
+        fillOpacity = stroke.opacity,
+        color = stroke.pal(mapfeat.stroke$stroke.features),
+        group = mapfeat.stroke$stroke.features
+      ) %>%
+      leaflet::addCircleMarkers(
+        lng = mapfeat.stroke$long,
+        lat = mapfeat.stroke$lat,
+        popup = mapfeat.df$link,
+        labelOptions = leaflet::labelOptions(
+          noHide = !(label.hide),
+          direction = label.position,
+          textOnly = TRUE,
+          style = list("font-size" = paste0(label.fsize, "px"))
+        ),
+        stroke = FALSE,
+        radius = 1.15 * width,
+        fillOpacity = opacity,
+        color = rev.stroke.pal(mapfeat.stroke$stroke.features),
+        group = mapfeat.stroke$stroke.features
+      )
+  }
 
-    # map: add points ----------------------------------------
-    if(density.points != FALSE){
-      m <- m %>% leaflet::addCircleMarkers(lng=mapfeat.df$long,
-                                         lat=mapfeat.df$lat,
-                                         popup= mapfeat.df$link,
-                                         stroke = FALSE,
-                                         radius = radius*1.1,
-                                         fillOpacity = opacity,
-                                         color = "black") %>%
-      leaflet::addCircleMarkers(lng=mapfeat.df$long,
-  lat=mapfeat.df$lat,
-  popup= mapfeat.df$link,
-  label= mapfeat.df$label,
-  labelOptions = leaflet::labelOptions(noHide = !(label.hide),
-                         direction = label.position,
-                         textOnly = TRUE,
-                         style = list("font-size" = paste0(label.fsize, "px"))),
-  stroke = FALSE,
-  radius = radius,
-  fillOpacity = opacity,
-  color = pal(mapfeat.df$features),
-  group = mapfeat.df$features)
+  # map: add points ----------------------------------------
+  if (density.points != FALSE & is.null(minichart)) {
+    m <- m %>% leaflet::addCircleMarkers(
+      lng = mapfeat.df$long,
+      lat = mapfeat.df$lat,
+      popup = mapfeat.df$link,
+      stroke = FALSE,
+      radius = width * 1.1,
+      fillOpacity = opacity,
+      color = "black",
+      group = mapfeat.df$features
+    ) %>%
+      leaflet::addCircleMarkers(
+        lng = mapfeat.df$long,
+        lat = mapfeat.df$lat,
+        popup = mapfeat.df$link,
+        stroke = FALSE,
+        radius = width,
+        fillOpacity = opacity,
+        color = pal(mapfeat.df$features),
+        group = mapfeat.df$features
+      )
+  }
+
+  # map: add minicharts -----------------------------------------------------
+
+  if (!is.null(minichart)) {
+    if (is.null(color)) {
+      color = my_colors
+    }
+
+    # create tables for each popup
+    tables <- ""
+    if (length(colnames(minichart.data)) > 1) {
+      new <-
+        as.data.frame(matrix(paste0(
+          rep(colnames(minichart.data), each = nrow(minichart.data)),
+          ": ",
+          as.matrix(minichart.data),
+          "<br> "
+        ), ncol = length(colnames(
+          minichart.data
+        ))))
+      df_args <- c(new, sep = "")
+      tables <- do.call(paste, df_args)
+    }
+
+    mapfeat.df$link <- paste0(mapfeat.df$link, tables)
+
+    m <- m %>% leaflet.minicharts::addMinicharts(
+      lng = mapfeat.df$long,
+      lat = mapfeat.df$lat,
+      chartdata = minichart.data,
+      type = minichart,
+      legend = legend,
+      width = 7 * width,
+      showLabels = minichart.labels,
+      popup = leaflet.minicharts::popupArgs(html = mapfeat.df$link),
+      time = minichart.time,
+      legendPosition = legend.position,
+      opacity = opacity,
+      colorPalette = color,
+      fillColor = color[1]
+    )
+  }
+
+  # map: labels -------------------------------------------------------------
+
+  if (density.points != FALSE & sum(label == "") != length(label)) {
+    m <- m %>% leaflet::addCircleMarkers(
+      lng = mapfeat.df$long,
+      lat = mapfeat.df$lat,
+      popup = mapfeat.df$link,
+      label = mapfeat.df$label,
+      labelOptions = leaflet::labelOptions(
+        noHide = !(label.hide),
+        direction = label.position,
+        textOnly = TRUE,
+        style = list("font-size" = paste0(label.fsize, "px"))
+      ),
+      stroke = FALSE,
+      radius = width * 3,
+      fillOpacity = 0,
+      color = "blue",
+      group = mapfeat.df$features
+    )
+    if ("emph" %in% colnames(mapfeat.df)) {
+      m <-
+        m %>% leaflet::addCircleMarkers(
+          lng = mapfeat.df[mapfeat.df$emph == "emph",]$long,
+          lat = mapfeat.df[mapfeat.df$emph == "emph",]$lat,
+          popup = mapfeat.df[mapfeat.df$emph == "emph",]$link,
+          label = mapfeat.df[mapfeat.df$emph == "emph",]$label,
+          labelOptions = leaflet::labelOptions(
+            noHide = !(label.hide),
+            direction = label.position,
+            textOnly = TRUE,
+            style = list(
+              "font-size" = paste0(label.fsize, "px"),
+              "color" = label.emphasize[[2]]
+            )
+          ),
+          stroke = FALSE,
+          radius = 3 * width,
+          fillOpacity = 0,
+          color = "red",
+          group = mapfeat.df$features
+        )
+    }
   }
 
   # map: images -------------------------------------------------------------
   if (!is.null(image.url)) {
-    m <- m %>% leaflet::addMarkers(lng=mapfeat.image$long,
-                             lat=mapfeat.image$lat,
-                             popup= mapfeat.image$link,
-                             icon = leaflet::icons(
-                               iconUrl = as.character(mapfeat.image$image.url),
-                               iconWidth = image.width,
-                               iconHeight = image.height,
-                               iconAnchorX = - image.X.shift,
-                               iconAnchorY = image.Y.shift))
+    m <- m %>% leaflet::addMarkers(
+      lng = mapfeat.image$long,
+      lat = mapfeat.image$lat,
+      popup = mapfeat.image$link,
+      icon = leaflet::icons(
+        iconUrl = as.character(mapfeat.image$image.url),
+        iconWidth = image.width,
+        iconHeight = image.height,
+        iconAnchorX = -image.X.shift,
+        iconAnchorY = image.Y.shift
+      )
+    )
   }
 
+
   # map: tile and control interaction --------------------------------------
-  if(length(tile) > 1){
+  if (length(tile) > 1) {
     if (control == TRUE) {
       m <- m %>% leaflet::addLayersControl(
         baseGroups = tile.name,
         overlayGroups = mapfeat.df$features,
-        options = leaflet::layersControlOptions(collapsed = FALSE))
+        options = leaflet::layersControlOptions(collapsed = FALSE)
+      )
+    } else if (density.control == TRUE) {
+      m <- m %>% leaflet::addLayersControl(
+        baseGroups = tile.name,
+        overlayGroups = my_poly_names,
+        options = leaflet::layersControlOptions(collapsed = FALSE)
+      )
     } else {
       m <- m %>% leaflet::addLayersControl(
         baseGroups = tile.name,
-        options = leaflet::layersControlOptions(collapsed = FALSE))
+        options = leaflet::layersControlOptions(collapsed = FALSE)
+      )
     }
   } else {
     if (control == TRUE) {
-      m <- m %>% leaflet::addLayersControl(overlayGroups = mapfeat.df$features,
-                     options = leaflet::layersControlOptions(collapsed = FALSE))
+      m <- m %>% leaflet::addLayersControl(
+        overlayGroups = mapfeat.df$features,
+        options = leaflet::layersControlOptions(collapsed = FALSE)
+      )
+    } else if (density.control == TRUE) {
+      m <- m %>% leaflet::addLayersControl(
+        overlayGroups = my_poly_names,
+        options = leaflet::layersControlOptions(collapsed = FALSE)
+      )
     }
   }
 
   # map: ScaleBar -----------------------------------------------------------
   if (scale.bar == TRUE) {
-    m <- m %>% leaflet::addScaleBar(
-      position = scale.bar.position)
+    m <- m %>% leaflet::addScaleBar(position = scale.bar.position)
   }
 
 
   # map: legend -------------------------------------------------------------
   if (sum(mapfeat.df$features == "") < length(mapfeat.df$features) &
-      legend == TRUE) {
-    m <- m %>% leaflet::addLegend(title = title,
-                                  position = legend.position,
-                                  pal = pal,
-                                  values = mapfeat.df$features,
-                                  opacity = legend.opacity)
+      legend == TRUE &
+      is.null(minichart)) {
+    m <- m %>% leaflet::addLegend(
+      title = title,
+      position = legend.position,
+      pal = pal,
+      values = mapfeat.df$features,
+      opacity = legend.opacity
+    )
   }
 
   # map: stroke.legend ------------------------------------------------------
   if (!is.null(stroke.features) & stroke.legend == TRUE) {
-    m <- m %>% leaflet::addLegend(title = stroke.title,
-                                  position = stroke.legend.position,
-                                  pal = stroke.pal,
-                                  values = mapfeat.stroke$stroke.features,
-                                  opacity = stroke.legend.opacity)
+    m <- m %>% leaflet::addLegend(
+      title = stroke.title,
+      position = stroke.legend.position,
+      pal = stroke.pal,
+      values = mapfeat.stroke$stroke.features,
+      opacity = stroke.legend.opacity
+    )
   }
 
   # map: density.legend ------------------------------------------------------
   if (!is.null(density.estimation) & density.legend == TRUE) {
-    m <- m %>% leaflet::addLegend(title = density.title,
-                                  position = density.legend.position,
-                                  pal = density.estimation.pal,
-                                  values = mapfeat.df$density.estimation,
-                                  opacity = density.legend.opacity)
+    m <- m %>% leaflet::addLegend(
+      title = density.title,
+      position = density.legend.position,
+      pal = density.estimation.pal,
+      values = mapfeat.df$density.estimation,
+      opacity = density.legend.opacity
+    )
   }
 
   # map: MiniMap ------------------------------------------------------------
@@ -481,7 +674,7 @@ leaflet::addCircleMarkers(lng=mapfeat.stroke$long,
   }
 
   # zoom.level argument -----------------------------------------------------
-  if(!is.null(zoom.level)) {
+  if (!is.null(zoom.level)) {
     m <- m %>% leaflet::setView(
       lng = mean(mapfeat.df$long),
       lat = mean(mapfeat.df$lat),
